﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SQLite;
using System.IO;
using System.Linq;
using System.Reflection;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using MyTools;

namespace MyToolsTestCsharp
{
    /// <summary>
    /// SQLiteDBHelperTest の概要の説明
    /// </summary>
    [TestClass]
    public class SQLiteHelperTest
    {
        private static string startupPath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
        private static string dbFile = "testdb.SQLiteHelperTest.sqlite3";
        private static string dbPath = Path.Combine(startupPath, dbFile);

        [TestInitialize()]
        public void SQLiteHelperTestInitialize()
        {
            var db = new SQLiteHelper(dbPath);
            db.DeleteDBFile();
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [Table2] ([Field1] TEXT, [Field2] TEXT)");
            db.ExecuteNonQuery("INSERT INTO [Table2] VALUES('text1', 'text1-1')");
            db.ExecuteNonQuery("INSERT INTO [Table2] VALUES('text2', 'text2-1')");
            db.ExecuteNonQuery("INSERT INTO [Table2] VALUES('text2', 'text2-2')");
        }

        [TestCleanup()]
        public void SQLiteHelperTestCleanup()
        {
            var db = new SQLiteHelper(dbPath);
            if (db.ExistDBFile())
            {
                try
                {
                    db.ExecuteNonQuery("DELETE FROM [Table2]");
                }
                catch { }
            }
        }

        [ClassCleanup()]
        public static void SQLiteHelperTestClassCleanup()
        {
            var db = new SQLiteHelper(dbPath);
            db.DeleteDBFile();
        }


        private TestContext testContextInstance;

        /// <summary>
        ///現在のテストの実行についての情報および機能を
        ///提供するテスト コンテキストを取得または設定します。
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region 追加のテスト属性
        //
        // テストを作成する際には、次の追加属性を使用できます:
        //
        // クラス内で最初のテストを実行する前に、ClassInitialize を使用してコードを実行してください
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // クラス内のテストをすべて実行したら、ClassCleanup を使用してコードを実行してください
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // 各テストを実行する前に、TestInitialize を使用してコードを実行してください
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // 各テストを実行した後に、TestCleanup を使用してコードを実行してください
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod, Description("Constractor : コンストラクタの引数がメンバ変数に格納される")]
        public void Cs_SQLiteHelper_Constractor()
        {
            // ↓ 実施
            var db = new SQLiteHelper(dbPath);

            // ↓ 結果確認
            ((string)db.AsDynamic().databasePath).Is(dbPath);
            ((string)db.AsDynamic().connectionString).Is("Data Source=" + dbPath);
        }



        [TestMethod, Description("ExistDBFile : DBファイルの存在確認(あり)")]
        public void Cs_SQLiteHelper_ExistDBFile_0001()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.CreateDB();

            // ↓ 実施 / 結果確認
            db.ExistDBFile().IsTrue();
        }

        [TestMethod, Description("ExistDBFile : DBファイルの存在確認(なし)")]
        public void Cs_SQLiteHelper_ExistDBFile_0002()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.DeleteDBFile();

            // ↓ 実施 / 結果確認
            db.ExistDBFile().IsFalse();
        }



        [TestMethod, Description("CreateDB : DBファイルの作成(既に存在する)")]
        public void Cs_SQLiteHelper_CreateDB_0001()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);

            // ↓ 実施 / 結果確認
            db.CreateDB().IsTrue();
        }

        [TestMethod, Description("CreateDB : DBファイルの作成(存在しない)")]
        public void Cs_SQLiteHelper_CreateDB_0002()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.DeleteDBFile();

            // ↓ 実施 / 結果確認
            db.CreateDB().IsTrue();
        }



        [TestMethod, Description("DeleteDBFile : DBファイルの削除(存在あり)")]
        public void Cs_SQLiteHelper_DeleteDBFile_0001()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);

            // ↓ 実施 / 結果確認
            db.DeleteDBFile().IsTrue();
        }

        [TestMethod, Description("DeleteDBFile : DBファイルの削除(存在なし)")]
        public void Cs_SQLiteHelper_DeleteDBFile_0002()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.DeleteDBFile();

            // ↓ 実施 / 結果確認
            db.DeleteDBFile().IsTrue();
        }



        [TestMethod, Description("ExistTable : DBファイルが存在するけど、テーブルが存在しない")]
        public void Cs_SQLiteHelper_ExistTable_0001()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);

            // ↓ 実施 / 結果確認
            db.ExistTable("TestTable1").IsFalse();
        }

        [TestMethod, Description("ExistTable : DBファイルが存在し、テーブルも存在する")]
        public void Cs_SQLiteHelper_ExistTable_0002()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [TestTable1] ([Field1] TEXT)");

            // ↓ 実施 / 結果確認
            db.ExistTable("TestTable1").IsTrue();
        }

        [TestMethod, Description("ExistTable : DBファイルが存在しない場合は Exception")]
        public void Cs_SQLiteHelper_ExistTable_0003()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.DeleteDBFile();

            // ↓ 実施 / 結果確認
            var ex = AssertEx.Throws<SQLiteException>(() =>
                db.ExistTable("TestTable1")
            );
            ex.ResultCode.Is(SQLiteErrorCode.CantOpen);
        }



        [TestMethod, Description("ExecuteNonQuery : パラメータなし")]
        public void Cs_SQLiteHelper_ExecuteNonQuery_0001()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [Table1] ([Field1] TEXT, [Field2] TEXT, [Field3] TEXT)");

            // ↓ 実施
            var count = db.ExecuteNonQuery("INSERT INTO [Table1] VALUES('パラメータなし', 'なし', null)");
            var result = db.ExecuteScalar("SELECT [Field2] FROM [Table1]");

            // ↓ 結果確認
            count.Is(1);
            result.Is("なし");
        }

        static object[] Cs_SQLiteHelper_ExecuteNonQuery_0002_TestSource = 
        {
            new object[] { DBNull.Value, "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", null, },
            new object[] { DBNull.Value, "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", new SQLiteParameter(), },
            new object[] { DBNull.Value, "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", new object[] { null }, },
            new object[] { DBNull.Value, "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", new object[] { new SQLiteParameter() }, },
            new object[] { "", "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", "", },
            new object[] { "", "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", new SQLiteParameter(DbType.String, (object)"")},
            new object[] { "", "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", new SQLiteParameter[] { new SQLiteParameter(DbType.String, (object)"")}, },
            new object[] { "1個", "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", "1個" },
            new object[] { "1個", "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", new SQLiteParameter(DbType.String, (object)"1個"), },
            new object[] { "1個", "INSERT INTO [Table1] VALUES('名前なしプレースホルダ 1個', ?, null)", new SQLiteParameter[] { new SQLiteParameter(DbType.String, (object)"1個")}, },
            new object[] { "1個", "INSERT INTO [Table1] VALUES('名前つきプレースホルダ 1個', @Field2, null)", new SQLiteParameter("@Field2", "1個"), },
        };
        [TestMethod, Description("ExecuteNonQuery : パラメータ 1個")]
        [TestCaseSource("Cs_SQLiteHelper_ExecuteNonQuery_0002_TestSource")]
        public void Cs_SQLiteHelper_ExecuteNonQuery_0002()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [Table1] ([Field1] TEXT, [Field2] TEXT, [Field3] TEXT)");

            TestContext.Run((object tcr, string tcs, object tcp) =>
            {
                // ↓ 実施
                db.ExecuteNonQuery("DELETE FROM [Table1]");
                var count = db.ExecuteNonQuery(tcs, tcp);
                var result = db.ExecuteScalar("SELECT [Field2] FROM [Table1]");

                // ↓ 結果確認
                count.Is(1);
                result.Is(tcr);
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteNonQuery_0003_TestSource = 
        {
            new object[] { DBNull.Value, null, null, },
            new object[] { DBNull.Value, null, new SQLiteParameter(), },
            new object[] { "", null, "", },
            new object[] { "", null, new SQLiteParameter(DbType.String, (object)"")},
            new object[] { "2個", null, "2個", },
            new object[] { "2個", null, new SQLiteParameter(DbType.String, (object)"2個"), },
        };
        [TestMethod, Description("ExecuteNonQuery : パラメータ 2個 個別指定")]
        [TestCaseSource("Cs_SQLiteHelper_ExecuteNonQuery_0003_TestSource")]
        public void Cs_SQLiteHelper_ExecuteNonQuery_0003()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [Table1] ([Field1] TEXT, [Field2] TEXT, [Field3] TEXT)");

            TestContext.Run((object tcr, string tcp1, object tcp2) =>
            {
                // ↓ 実施
                db.ExecuteNonQuery("DELETE FROM [Table1]");
                var count = db.ExecuteNonQuery("INSERT INTO [Table1] VALUES('名前なしプレースホルダ 2個 個別指定', ?, ?)", tcp1, tcp2);
                var result = db.ExecuteScalar("SELECT [Field3] FROM [Table1]");

                // ↓ 結果確認
                count.Is(1);
                result.Is(tcr);
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteNonQuery_0004_TestSource = 
        {
            new object[] { DBNull.Value, new object[] { null, null }, },
            new object[] { DBNull.Value, new object[] { null, new SQLiteParameter() }, },
            new object[] { "", new SQLiteParameter[] { null, new SQLiteParameter(DbType.String, (object)"")}, },
            new object[] { "2個", new SQLiteParameter[] { null, new SQLiteParameter(DbType.String, (object)"2個")}, },
        };
        [TestMethod, Description("ExecuteNonQuery : パラメータ 2個 配列指定")]
        [TestCaseSource("Cs_SQLiteHelper_ExecuteNonQuery_0004_TestSource")]
        public void Cs_SQLiteHelper_ExecuteNonQuery_0004()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [Table1] ([Field1] TEXT, [Field2] TEXT, [Field3] TEXT)");

            TestContext.Run((object tcr, object tcp) =>
            {
                // ↓ 実施
                db.ExecuteNonQuery("DELETE FROM [Table1]");
                var count = db.ExecuteNonQuery("INSERT INTO [Table1] VALUES('名前なしプレースホルダ 2個 配列指定', ?, ?)", tcp);
                var result = db.ExecuteScalar("SELECT [Field3] FROM [Table1]");

                // ↓ 結果確認
                count.Is(1);
                result.Is(tcr);
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteNonQuerys_0001_TestSource = 
        {
            new object[] { DBNull.Value, null, },
            new object[] { DBNull.Value, new object[][] {}, },
            new object[] { DBNull.Value, new object[][] { null }, },
            new object[] { DBNull.Value, new object[][] { new object[] { }, }, },
            new object[] { DBNull.Value, new object[][] { new SQLiteParameter[] { }, }, },
            new object[] { DBNull.Value, new object[][] { new object[] { }, new object[] { }, }, },
            new object[] { DBNull.Value, new object[][] { new object[] { }, new SQLiteParameter[] { }, }, },
            new object[] { DBNull.Value, new object[][] { new SQLiteParameter[] { }, new SQLiteParameter[] { }, }, },
            new object[] { DBNull.Value, new SQLiteParameter[][] {}, },
            new object[] { DBNull.Value, new SQLiteParameter[][] { null }, },
            new object[] { DBNull.Value, new SQLiteParameter[][] { new SQLiteParameter[] { }, }, },
            new object[] { DBNull.Value, new SQLiteParameter[][] { new SQLiteParameter[] { }, new SQLiteParameter[] { }, }, },
        };
        [TestMethod, Description("ExecuteNonQuerys : プレースホルダなし")]
        [TestCaseSource("Cs_SQLiteHelper_ExecuteNonQuerys_0001_TestSource")]
        public void Cs_SQLiteHelper_ExecuteNonQuerys_0001()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [Table1] ([Field1] TEXT, [Field2] TEXT, [Field3] TEXT)");

            TestContext.Run((object tcr, object tcp) =>
            {
                // ↓ 実施 / 結果確認
                var ex = AssertEx.Throws<SQLiteException>(() =>
                    db.ExecuteNonQuerys("INSERT INTO [Table1] VALUES('プレースホルダなし', ?, null)", (object[][])tcp)
                    );
                ex.Message.Is("unknown error\r\nInsufficient parameters supplied to the command");
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteNonQuerys_0002_TestSource = 
        {
            new object[] { DBNull.Value, new [] { new [] { "a", "2", }, new [] { null, "1", }, }, },
            new object[] { "",           new [] { new [] { "a", "2", }, new [] { "", "1", }, }, },
            new object[] { "2個",        new [] { new [] { "a", "2", }, new [] { "2個", "1", }, }, },
            new object[] { DBNull.Value, new [] { 
                new [] { new SQLiteParameter(DbType.String, (object)"a"),   new SQLiteParameter(DbType.String, (object)"2"), }, 
                new [] { null,                                              new SQLiteParameter(DbType.String, (object)"1"), }, }, },
            new object[] { DBNull.Value, new [] { 
                new [] { new SQLiteParameter(DbType.String, (object)"a"),   new SQLiteParameter(DbType.String, (object)"2"), }, 
                new [] { new SQLiteParameter(),                             new SQLiteParameter(DbType.String, (object)"1"), }, }, },
            new object[] { "", new [] { 
                new [] { new SQLiteParameter(DbType.String, (object)"a"),   new SQLiteParameter(DbType.String, (object)"2"), }, 
                new [] { new SQLiteParameter(DbType.String, (object)""),    new SQLiteParameter(DbType.String, (object)"1"), }, }, },
            new object[] { "2個", new [] { 
                new [] { new SQLiteParameter(DbType.String, (object)"a"),   new SQLiteParameter(DbType.String, (object)"2"), }, 
                new [] { new SQLiteParameter(DbType.String, (object)"2個"), new SQLiteParameter(DbType.String, (object)"1"), }, }, },
        };
        [TestMethod, Description("ExecuteNonQuerys : 名前なしプレースホルダ 1個")]
        [TestCaseSource("Cs_SQLiteHelper_ExecuteNonQuerys_0002_TestSource")]
        public void Cs_SQLiteHelper_ExecuteNonQuerys_0002()
        {
            // ↓ 準備
            var db = new SQLiteHelper(dbPath);
            db.ExecuteNonQuery(@"CREATE TABLE IF NOT EXISTS [Table1] ([Field1] TEXT, [Field2] TEXT, [Field3] TEXT)");

            TestContext.Run((object tcr, object tcp) =>
            {
                // ↓ 実施
                db.ExecuteNonQuery("DELETE FROM [Table1]");
                var count = db.ExecuteNonQuerys("INSERT INTO [Table1] VALUES('名前なしプレースホルダ 2個', ?, ?)", (object[][])tcp);
                var result = db.ExecuteScalar("SELECT [Field2] FROM [Table1] WHERE [Field3] = '1'");

                // ↓ 結果確認
                count.Is(2);
                result.Is(tcr);
            });
        }



        static object[] Cs_SQLiteHelper_ExecuteReader_0001_TestSource = 
        {
            new object[] { new string[][] { }, 
                           "SELECT * FROM [Table2] WHERE [Field1]='text0'", }, 
            new object[] { new string[][] { new [] { "text1", "text1-1" }, }, 
                           "SELECT * FROM [Table2] WHERE [Field1]='text1'", }, 
            new object[] { new string[][] { new [] { "text2", "text2-1" }, new [] { "text2", "text2-2" } }, 
                           "SELECT * FROM [Table2] WHERE [Field1]='text2'", }, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteReader_0001_TestSource")]
        [TestMethod, Description("ExecuteReader : パラメータなし")]
        public void Cs_SQLiteHelper_ExecuteReader_0001()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcp) =>
            {
                // ↓ 実施
                using (var reader = db.ExecuteReader(tcp))
                {
                    // ↓ 結果確認
                    var result = readerToString2DArray(reader);
                    result.IsStructuralEqual(tcr, ToString(result));
                }
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteReader_0002_TestSource = 
        {
            new object[] { new string[][] { }, "SELECT * FROM [Table2] WHERE [Field1]='text0'", null, }, 
            new object[] { new string[][] { }, "SELECT * FROM [Table2] WHERE [Field1]='text0'", new object[] {}, }, 
            new object[] { new string[][] { }, "SELECT * FROM [Table2] WHERE [Field1]='text0'", new object[] { null }, }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=?", "text1"}, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=?", new SQLiteParameter(DbType.String, (object)"text1")}, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=@Field1", new SQLiteParameter("@Field1", "text1")}, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteReader_0002_TestSource")]
        [TestMethod, Description("ExecuteReader : パラメータ 1個")]
        public void Cs_SQLiteHelper_ExecuteReader_0002()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcs, object tcp) =>
            {
                // ↓ 実施
                using (var reader = db.ExecuteReader(tcs, tcp))
                {
                    // ↓ 結果確認
                    var result = readerToString2DArray(reader);
                    result.IsStructuralEqual(tcr, ToString(result));
                }
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteReader_0003_TestSource = 
        {
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", "text1", "text1-1", }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", new SQLiteParameter(DbType.String, (object)"text1"), new SQLiteParameter(DbType.String, (object)"text1-1"), }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=@Field1 and [Field2]=@Field2", new SQLiteParameter("@Field1", "text1"), new SQLiteParameter("@Field2", "text1-1"), }, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteReader_0003_TestSource")]
        [TestMethod, Description("ExecuteReader : パラメータ 2個 個別指定")]
        public void Cs_SQLiteHelper_ExecuteReader_0003()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcs, object tcp1, object tcp2) =>
            {
                using (var reader = db.ExecuteReader(tcs, tcp1, tcp2))
                {
                    // ↓ 結果確認
                    var result = readerToString2DArray(reader);
                    result.IsStructuralEqual(tcr, ToString(result));
                }
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteReader_0004_TestSource = 
        {
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", new object[] { "text1", "text1-1", }, }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", new SQLiteParameter[] { new SQLiteParameter(DbType.String, (object)"text1"), new SQLiteParameter(DbType.String, (object)"text1-1"), }, }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=@Field1 and [Field2]=@Field2", new SQLiteParameter[] { new SQLiteParameter("@Field1", "text1"), new SQLiteParameter("@Field2", "text1-1"), }, },
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteReader_0004_TestSource")]
        [TestMethod, Description("ExecuteReader : パラメータ 2個 配列指定")]
        public void Cs_SQLiteHelper_ExecuteReader_0004()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcs, object tcp) =>
            {
                // ↓ 実施
                using (var reader = db.ExecuteReader(tcs, tcp))
                {
                    // ↓ 結果確認
                    var result = readerToString2DArray(reader);
                    result.IsStructuralEqual(tcr, ToString(result));
                }
            });
        }



        static object[] Cs_SQLiteHelper_ExecuteDataTable_0001_TestSource = 
        {
            new object[] { new string[][] { }, 
                           "SELECT * FROM [Table2] WHERE [Field1]='text0'", }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, 
                           "SELECT * FROM [Table2] WHERE [Field1]='text1'", }, 
            new object[] { new string[][] { new string[] { "text2", "text2-1" }, new string[] { "text2", "text2-2" } }, 
                           "SELECT * FROM [Table2] WHERE [Field1]='text2'", }, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteDataTable_0001_TestSource")]
        [TestMethod, Description("ExecuteDataTable : パラメータなし")]
        public void Cs_SQLiteHelper_ExecuteDataTable_0001()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcs) =>
            {
                // ↓ 実施
                var dt = db.ExecuteDataTable(tcs);

                // ↓ 結果確認
                var result = dt.Rows.Cast<DataRow>().Select(r => r.ItemArray.Select(c => c.ToString()).ToArray()).ToArray();
                result.IsStructuralEqual(tcr, ToString(result));
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteDataTable_0002_TestSource = 
        {
            new object[] { new string[][] { }, "SELECT * FROM [Table2] WHERE [Field1]='text0'", null, }, 
            new object[] { new string[][] { }, "SELECT * FROM [Table2] WHERE [Field1]='text0'", new object[] {}, }, 
            new object[] { new string[][] { }, "SELECT * FROM [Table2] WHERE [Field1]='text0'", new object[] { null }, }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=?", "text1"}, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=?", new SQLiteParameter(DbType.String, (object)"text1")}, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=@Field1", new SQLiteParameter("@Field1", "text1")}, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteDataTable_0002_TestSource")]
        [TestMethod, Description("ExecuteDataTable : パラメータ 1個")]
        public void Cs_SQLiteHelper_ExecuteDataTable_0002()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcs, object tcp) =>
            {
                // ↓ 実施
                var dt = db.ExecuteDataTable(tcs, tcp);

                // ↓ 結果確認
                var result = dt.Rows.Cast<DataRow>().Select(r => r.ItemArray.Select(c => c.ToString()).ToArray()).ToArray();
                result.IsStructuralEqual(tcr, ToString(result));
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteDataTable_0003_TestSource = 
        {
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", "text1", "text1-1", }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", new SQLiteParameter(DbType.String, (object)"text1"), new SQLiteParameter(DbType.String, (object)"text1-1"), }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=@Field1 and [Field2]=@Field2", new SQLiteParameter("@Field1", "text1"), new SQLiteParameter("@Field2", "text1-1"), }, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteDataTable_0003_TestSource")]
        [TestMethod, Description("ExecuteDataTable : パラメータ 2個 個別指定")]
        public void Cs_SQLiteHelper_ExecuteDataTable_0003()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcs, object tcp1, object tcp2) =>
            {
                // ↓ 実施
                var dt = db.ExecuteDataTable(tcs, tcp1, tcp2);

                // ↓ 結果確認
                var result = dt.Rows.Cast<DataRow>().Select(r => r.ItemArray.Select(c => c.ToString()).ToArray()).ToArray();
                result.IsStructuralEqual(tcr, ToString(result));
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteDataTable_0004_TestSource = 
        {
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", new object[] { "text1", "text1-1", }, }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=? and [Field2]=?", new SQLiteParameter[] { new SQLiteParameter(DbType.String, (object)"text1"), new SQLiteParameter(DbType.String, (object)"text1-1"), }, }, 
            new object[] { new string[][] { new string[] { "text1", "text1-1" } }, "SELECT * FROM [Table2] WHERE [Field1]=@Field1 and [Field2]=@Field2", new SQLiteParameter[] { new SQLiteParameter("@Field1", "text1"), new SQLiteParameter("@Field2", "text1-1"), }, },
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteDataTable_0004_TestSource")]
        [TestMethod, Description("ExecuteDataTable : パラメータ 2個 配列指定")]
        public void Cs_SQLiteHelper_ExecuteDataTable_0004()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((string[][] tcr, string tcs, object tcp) =>
            {
                // ↓ 実施
                var dt = db.ExecuteDataTable(tcs, tcp);
                var result = dt.Rows.Cast<DataRow>().Select(r => r.ItemArray.Select(c => c.ToString()).ToArray()).ToArray();

                // ↓ 結果確認
                result.IsStructuralEqual(tcr, ToString(result));
            });
        }



        static object[] Cs_SQLiteHelper_ExecuteScalar_0001_TestSource = 
        {
            new object[] { 0, "SELECT count(*) FROM [Table2] WHERE [Field1]='text0'", }, 
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]='text1'", }, 
            new object[] { 2, "SELECT count(*) FROM [Table2] WHERE [Field1]='text2'", }, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteScalar_0001_TestSource")]
        [TestMethod, Description("ExecuteScalar : パラメータなし")]
        public void Cs_SQLiteHelper_ExecuteScalar_0001()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((int tcr, string tcs) =>
            {
                // ↓ 実施
                var result = Convert.ToInt32(db.ExecuteScalar(tcs));

                // ↓ 結果確認
                result.Is(tcr);
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteScalar_0002_TestSource = 
        {
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]='text1'", null, }, 
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]='text1'", new object[] {}, }, 
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]='text1'", new object[] { null }, }, 
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]=?", "text1", }, 
            new object[] { 2, "SELECT count(*) FROM [Table2] WHERE [Field1]=?", new SQLiteParameter(DbType.String, (object)"text2")}, 
            new object[] { 2, "SELECT count(*) FROM [Table2] WHERE [Field1]=@Field1", new SQLiteParameter("@Field1", "text2")}, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteScalar_0002_TestSource")]
        [TestMethod, Description("ExecuteScalar : パラメータ 1個")]
        public void Cs_SQLiteHelper_ExecuteScalar_0002()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((int tcr, string tcs, object tcp) =>
            {
                // ↓ 実施
                var result = Convert.ToInt32(db.ExecuteScalar(tcs, tcp));

                // ↓ 結果確認
                result.Is(tcr);
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteScalar_0003_TestSource = 
        {
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]=? and [Field2]=?", "text1", "text1-1", }, 
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]=? and [Field2]=?", new SQLiteParameter(DbType.String, (object)"text2"), new SQLiteParameter(DbType.String, (object)"text2-1"), }, 
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]=@Field1 and [Field2]=@Field2", new SQLiteParameter("@Field1", "text2"), new SQLiteParameter("@Field2", "text2-2"), }, 
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteScalar_0003_TestSource")]
        [TestMethod, Description("ExecuteScalar : パラメータ 2個 個別指定")]
        public void Cs_SQLiteHelper_ExecuteScalar_0003()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((int tcr, string tcs, object tcp1, object tcp2) =>
            {
                // ↓ 実施
                var result = Convert.ToInt32(db.ExecuteScalar(tcs, tcp1, tcp2));

                // ↓ 結果確認
                result.Is(tcr);
            });
        }

        static object[] Cs_SQLiteHelper_ExecuteScalar_0004_TestSource = 
        {
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]=? and [Field2]=?", new object[] {"text1", "text1-1", }, },
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]=? and [Field2]=?", new SQLiteParameter[] { new SQLiteParameter(DbType.String, (object)"text2"), new SQLiteParameter(DbType.String, (object)"text2-1"), }, },
            new object[] { 1, "SELECT count(*) FROM [Table2] WHERE [Field1]=@Field1 and [Field2]=@Field2", new SQLiteParameter[] { new SQLiteParameter("@Field1", "text2"), new SQLiteParameter("@Field2", "text2-2"), }, },
        };
        [TestCaseSource("Cs_SQLiteHelper_ExecuteScalar_0004_TestSource")]
        [TestMethod, Description("ExecuteScalar : パラメータ 2個 個別指定")]
        public void Cs_SQLiteHelper_ExecuteScalar_0004()
        {
            var db = new SQLiteHelper(dbPath);
            TestContext.Run((int tcr, string tcs, object tcp) =>
            {
                // ↓ 実施
                var result = Convert.ToInt32(db.ExecuteScalar(tcs, tcp));

                // ↓ 結果確認
                result.Is(tcr);
            });
        }



        public string[][] readerToString2DArray(SQLiteDataReader reader)
        {
            var oa = new List<string[]>();
            while (reader.Read())
            {
                var values = new object[reader.FieldCount];
                var fieldCount = reader.GetValues(values);
                var ia = values.Select(x => x.ToString()).ToArray();

                oa.Add(ia);
            }
            return oa.ToArray();
        }

        public string ToString(string[][] a)
        {
            int lenD1 = a.Length;
            int lenD2 = a.Length > 0 ? a[0].Length : 0;
            string text = "";

            for (int i = 0; i < lenD1; i++)
            {
                text += "{";
                for (int j = 0; j < lenD2; j++)
                {
                    text += a[i][j];
                    text += ",";
                }
                text += "},";
            }
            return string.Format("[{0}][{1}] {2}", lenD1, lenD2, text);
        }
    }
}
